@@ -1,6 +1,6 @@
 ;;; sb-asahi.el --- shimbun backend for asahi.com -*- coding: iso-2022-7bit; -*-
 
-;; Copyright (C) 2001, 2002, 2003 Yuuichi Teranishi <teranisi@gohome.org>
+;; Copyright (C) 2001, 2002, 2003, 2004 Yuuichi Teranishi <teranisi@gohome.org>
 
 ;; Author: TSUCHIYA Masatoshi <tsuchiya@namazu.org>,
 ;;         Yuuichi Teranishi  <teranisi@gohome.org>,
@@ -45,7 +45,7 @@
   "Name of the parent url.")
 
 (defvar shimbun-asahi-group-table
-  (let* ((s0 "[\t\n ]*")
+  (let* ((s0 "[\t\n $B!!(B]*")
 	 (s1 "[\t\n ]+")
 	 (no-nl "[^\n<>]+")
 	 (default (list
@@ -73,7 +73,25 @@
 		    ;; 9. hour:minute
 		    "\\([012][0-9]:[0-5][0-9]\\)"
 		    "\\)?)")
-		   1 3 4 5 6 7 9)))
+		   1 3 4 5 6 7 9))
+	 (antarctica (list
+		      (concat
+		       "<a" s1 "href=\""
+		       ;; 1. url
+		       "\\(/nankyoku/%s/"
+		       ;; 2. serial number
+		       "\\([A-Z0-9]+\\)"
+		       "\\.html\\)\">" s0
+		       ;; 3. subject
+		       "\\(" no-nl "\\)" s0 "</a>" s0
+		       "[$B!J(B(][0-9]+/"
+		       ;; 4. month
+		       "\\([01][0-9]\\)"
+		       "/"
+		       ;; 5. day
+		       "\\([0-3][0-9]\\)"
+		       "[)$B!K(B]")
+		      1 nil 2 3 4 5)))
     `(("business" "$B7P:Q(B" "%s/" ,@default)
       ("culture" "$BJ82=!&7]G=(B" "%s/" ,@default)
       ("english" "ENGLISH" "%s/"
@@ -143,7 +161,27 @@
 	 ;; 11. hour:minute
 	 "\\([012][0-9]:[0-5][0-9]\\)"
 	 "\\)?)")
-       1 nil 6 7 8 9 11 3 5)))
+       1 nil 6 7 8 9 11 3 5)
+      ("nankyoku" "$BFn6K(B" "%s/news/index.html"
+       ,(concat
+	 "<a" s1 "href=\""
+	 ;; 1. url
+	 "\\(/%s/news/"
+	 ;; 2. serial number
+	 "\\([A-Z0-9]+\\)"
+	 "\\.html\\)\">" s0
+	 ;; 3. subject
+	 "\\(" no-nl "\\)" s0 "</a>" s0
+	 "[$B!J(B(][0-9]+/"
+	 ;; 4. month
+	 "\\([01][0-9]\\)"
+	 "/"
+	 ;; 5. day
+	 "\\([0-3][0-9]\\)"
+	 "[)$B!K(B]")
+       1 nil 2 3 4 5)
+      ("whitemail" "WhiteMail$B!wFn6K(B" "nankyoku/%s/" ,@antarctica)
+      ("borderless" "$B9q6-$N$J$$BgN&$+$i(B" "nankyoku/%s/" ,@antarctica)))
   "Alist of group names, their Japanese translations, index pages,
 regexps and numbers.  Where index pages and regexps may contain the
 \"%s\" token which is replaced with group names, numbers point to the
@@ -210,7 +248,7 @@ bIy3rr^<Q#lf&~ADU:X!t5t>gW5)Q]N{Mmn\n L]suPpL|gFjV{S|]a-:)\\FR\
 					  nil t)
 		      (match-string 1))))))
       (setq month (string-to-number (match-string (nth 4 numbers)))
-	    year (cond ((and (= 12 month) (= 1 cmonth))
+	    year (cond ((>= (- month cmonth) 2)
 			(1- cyear))
 		       ((and (= 1 month) (= 12 cmonth))
 			(1+ cyear))
