@@ -1,6 +1,6 @@
 ;;; shimbun.el --- interfacing with web newspapers -*- coding: iso-2022-7bit; -*-
 
-;; Copyright (C) 2001, 2002, 2003 Yuuichi Teranishi <teranisi@gohome.org>
+;; Copyright (C) 2001, 2002, 2003, 2004 Yuuichi Teranishi <teranisi@gohome.org>
 
 ;; Author: TSUCHIYA Masatoshi <tsuchiya@namazu.org>,
 ;;         Akihiro Arisawa    <ari@mbf.sphere.ne.jp>,
@@ -161,11 +161,11 @@ while shimbun is waiting for a server's response."
 
 ;;; emacs-w3m implementation of url retrieval and entity decoding.
 (require 'w3m)
-(defun shimbun-retrieve-url (url &optional no-cache no-decode)
+(defun shimbun-retrieve-url (url &optional no-cache no-decode referer)
   "Rertrieve URL contents and insert to current buffer.
 Return content-type of URL as string when retrieval succeeded."
   (let (type)
-    (when (and url (setq type (w3m-retrieve url nil no-cache)))
+    (when (and url (setq type (w3m-retrieve url nil no-cache nil referer)))
       (unless no-decode
 	(w3m-decode-buffer url)
 	(goto-char (point-min)))
@@ -512,7 +512,7 @@ entities."
 	    (save-match-data
 	      (with-temp-buffer
 		(set-buffer-multibyte nil)
-		(let ((type (shimbun-retrieve-url url nil t)))
+		(let ((type (shimbun-retrieve-url url nil t base-url)))
 		  (when type
 		    (push (setq img
 				(cons url
@@ -552,25 +552,6 @@ content-type if `shimbun-encapsulate-images' is non-nil."
       (insert "MIME-Version: 1.0\n")
       (shimbun-entity-insert body))))
 
-(defsubst shimbun-make-html-contents (shimbun header)
-  (let ((case-fold-search t)
-	start)
-    (when (and (re-search-forward (shimbun-content-start-internal shimbun)
-				  nil t)
-	       (setq start (point))
-	       (re-search-forward (shimbun-content-end-internal shimbun)
-				  nil t))
-      (delete-region (match-beginning 0) (point-max))
-      (delete-region (point-min) start)
-      (goto-char (point-min))
-      (insert "<html>\n<head>\n<base href=\""
-	      (shimbun-header-xref header) "\">\n</head>\n<body>\n")
-      (goto-char (point-max))
-      (insert (shimbun-footer shimbun header t)
-	      "\n</body>\n</html>\n"))
-    (shimbun-make-mime-article shimbun header)
-    (buffer-string)))
-
 (defcustom shimbun-x-face-database-function
   (if (boundp 'shimbun-use-bbdb-for-x-face)
       (cdr (assq (symbol-value 'shimbun-use-bbdb-for-x-face)
@@ -788,13 +769,12 @@ Return nil if all pages should be retrieved."
   "Return a X-Face field string for SHIMBUN.")
 
 (luna-define-method shimbun-x-face ((shimbun shimbun))
-  (or (shimbun-x-face-internal shimbun)
-      (shimbun-set-x-face-internal
-       shimbun
-       (or (cdr (assoc (shimbun-current-group-internal shimbun)
-		       (shimbun-x-face-alist-internal shimbun)))
-	   (cdr (assoc "default" (shimbun-x-face-alist-internal shimbun)))
-	   shimbun-x-face))))
+  (shimbun-set-x-face-internal
+   shimbun
+   (or (cdr (assoc (shimbun-current-group-internal shimbun)
+		   (shimbun-x-face-alist-internal shimbun)))
+       (cdr (assoc "default" (shimbun-x-face-alist-internal shimbun)))
+       shimbun-x-face)))
 
 (defun shimbun-search-id (shimbun id)
   "Return non-nil when MUA found a message structure which corresponds to ID."
@@ -837,9 +817,37 @@ If OUTBUF is not specified, article is retrieved to the current buffer.")
   "Return a content string of SHIMBUN article using current buffer content.
 HEADER is a header structure obtained via `shimbun-headers'.")
 
+(defsubst shimbun-make-html-contents (shimbun header)
+  (when (shimbun-clear-contents shimbun header)
+    (goto-char (point-min))
+    (insert "<html>\n<head>\n<base href=\""
+	    (shimbun-header-xref header)
+	    "\">\n</head>\n<body>\n")
+    (goto-char (point-max))
+    (insert (shimbun-footer shimbun header t)
+	    "\n</body>\n</html>\n"))
+  (shimbun-make-mime-article shimbun header)
+  (buffer-string))
+
 (luna-define-method shimbun-make-contents ((shimbun shimbun) header)
   (shimbun-make-html-contents shimbun header))
 
+(luna-define-generic shimbun-clear-contents (shimbun header)
+  "Clear a content in this current buffer for an article of SHIMBUN.
+Return nil, unless a content is cleared successfully.")
+
+(luna-define-method shimbun-clear-contents ((shimbun shimbun) header)
+  (let ((case-fold-search t) start)
+    (goto-char (point-min))
+    (when (and (re-search-forward (shimbun-content-start-internal shimbun)
+				  nil t)
+	       (setq start (point))
+	       (re-search-forward (shimbun-content-end-internal shimbun)
+				  nil t))
+      (delete-region (match-beginning 0) (point-max))
+      (delete-region (point-min) start)
+      t)))
+
 (luna-define-generic shimbun-footer (shimbun header &optional html)
   "Make a footer string for SHIMBUN and HEADER.")
 
@@ -881,6 +889,32 @@ integer n:    Retrieve n pages of header indices.")
 	    " $B$G8x3+$5$l$F$$$^$9!#(B\n")))
 
 ;;; Misc Functions
+(static-cond
+ ((fboundp 'point-at-bol)
+  (defalias 'shimbun-point-at-bol 'point-at-bol))
+ ((fboundp 'line-beginning-position)
+  (defalias 'shimbun-point-at-bol 'line-beginning-position))
+ (t
+  (defun shimbun-point-at-bol ()
+    "Return point at the beginning of the line."
+    (let ((p (point)))
+      (beginning-of-line)
+      (prog1 (point)
+	(goto-char p))))))
+
+(static-cond
+ ((fboundp 'point-at-eol)
+  (defalias 'shimbun-point-at-eol 'point-at-eol))
+ ((fboundp 'line-end-position)
+  (defalias 'shimbun-point-at-eol 'line-end-position))
+ (t
+  (defun shimbun-point-at-eol ()
+    "Return point at the end of the line."
+    (let ((p (point)))
+      (end-of-line)
+      (prog1 (point)
+	(goto-char p))))))
+
 (defun shimbun-header-insert-and-buffer-string (shimbun header
 							&optional charset html)
   "Insert headers which are generated from SHIMBUN and HEADER, and
@@ -919,20 +953,30 @@ return the contents of this buffer as an encoded string."
     (error string)))
 
 (defun shimbun-make-date-string (year month day &optional time timezone)
-  (format "%02d %s %04d %s %s"
-	  day
-	  (aref [nil "Jan" "Feb" "Mar" "Apr" "May" "Jun"
-		     "Jul" "Aug" "Sep" "Oct" "Nov" "Dec"]
-		month)
-	  (cond ((< year 69)
-		 (+ year 2000))
-		((< year 100)
-		 (+ year 1900))
-		((< year 1000)	; possible 3-digit years.
-		 (+ year 1900))
-		(t year))
-	  (or time "00:00")
-	  (or timezone "+0900")))
+  "Make a date string which will be used as shimbun headers.
+
+YEAR is a 4-digit number, and MONTH and DAY are also numbers.  TIME is
+a string in the \"HH:MM\" form, where HH is a number of hours and MM is
+a number of minutes.  It defaults to \"00:00\".
+
+TIMEZONE defaults to \"+0900\" by the historical reason.  You should
+specify this if a time is represented based on other than the \"+0900\"
+zone."
+  (setq year (cond ((< year 69)
+		    (+ year 2000))
+		   ((< year 100)
+		    (+ year 1900))
+		   ((< year 1000)	; possible 3-digit years.
+		    (+ year 1900))	; why isn't it 1000?
+		   (t year)))
+  (let ((cts (current-time-string (encode-time 0 0 0 day month year))))
+    (format "%s, %02d %s %04d %s %s"
+	    (substring cts 0 3)
+	    day
+	    (substring cts 4 7)
+	    year
+	    (or time "00:00")
+	    (or timezone "+0900"))))
 
 (autoload 'timezone-fix-time "timezone")
 
@@ -968,6 +1012,19 @@ is enclosed by at least one regexp grouping construct."
     (shimbun-decode-entities)
     (buffer-string)))
 
+(defun shimbun-remove-tags (begin-tag &optional end-tag)
+  "Remove all occurrences of regions surrounded by BEGIN-TAG and END-TAG."
+  (let ((case-fold-search t))
+    (goto-char (point-min))
+    (if end-tag
+	(let (pos)
+	  (while (and (re-search-forward begin-tag nil t)
+		      (setq pos (match-beginning 0))
+		      (re-search-forward end-tag nil t))
+	    (delete-region pos (point))))
+      (while (re-search-forward begin-tag nil t)
+	(delete-region (match-beginning 0) (match-end 0))))))
+
 (defun shimbun-remove-markup ()
   "Remove all HTML markup, leaving just plain text."
   (save-excursion
@@ -980,6 +1037,12 @@ is enclosed by at least one regexp grouping construct."
     (while (re-search-forward "<[^>]+>" nil t)
       (replace-match "" t t))))
 
+(defsubst shimbun-strip-cr ()
+  "Strip ^M from the end of all lines."
+  (goto-char (point-max))
+  (while (search-backward "\r\n" nil t)
+    (delete-char 1)))
+
 (eval-and-compile
   (cond
    ((fboundp 'replace-in-string)
@@ -1017,28 +1080,36 @@ Use ## to put a single # into the output.  If `shimbun-verbose' is nil,
 it will run silently.  The `shimbun-message-enable-logging' variable
 controls whether this function should preserve a message in the
 *Messages* buffer."
-  (let (rest server string case-fold-search)
-    (when (string-match "#[#gsS]" fmt)
-      (setq rest (shimbun-replace-in-string fmt "##" "#")
-	    rest (shimbun-replace-in-string
-		  rest "#g" (shimbun-current-group-internal shimbun))
-	    rest (shimbun-replace-in-string
-		  rest "#s" (setq server (shimbun-server-internal shimbun)))
-	    fmt (shimbun-replace-in-string
-		 rest "#S" (or (shimbun-server-name-internal shimbun)
-			       server))))
-    (if shimbun-verbose
-	(static-if (featurep 'xemacs)
-	    (prog1
-		(setq string (apply 'format fmt args))
-	      (if shimbun-message-enable-logging
-		  (display-message 'message string)
-		(display-message 'no-log string)))
-	  (if shimbun-message-enable-logging
-	      (apply 'message fmt args)
-	    (let (message-log-max)
-	      (apply 'message fmt args))))
-      (apply 'format fmt args))))
+  (let ((default-enable-multibyte-characters t)
+	specifier)
+    (with-temp-buffer
+      (insert fmt)
+      (goto-char (point-min))
+      (while (search-forward "#" nil t)
+	(setq specifier (char-after))
+	(delete-region (1- (point)) (1+ (point)))
+	(cond ((eq specifier ?#)
+	       (insert "#"))
+	      ((eq specifier ?g)
+	       (insert (shimbun-current-group-internal shimbun)))
+	      ((eq specifier ?s)
+	       (insert (shimbun-server-internal shimbun)))
+	      ((eq specifier ?S)
+	       (insert (or (shimbun-server-name-internal shimbun)
+			   (shimbun-server-internal shimbun))))))
+      (setq fmt (buffer-string))))
+  (if shimbun-verbose
+      (static-if (featurep 'xemacs)
+	  (let ((string (apply 'format fmt args)))
+	    (if shimbun-message-enable-logging
+		(display-message 'message string)
+	      (display-message 'no-log string))
+	    string)
+	(if shimbun-message-enable-logging
+	    (apply 'message fmt args)
+	  (let (message-log-max)
+	    (apply 'message fmt args))))
+    (apply 'format fmt args)))
 
 (provide 'shimbun)
 
